package org.ow2.bonita.example.aw;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import org.ow2.bonita.example.AbstractExample;
import org.ow2.bonita.facade.QueryDefinitionAPI;
import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.RuntimeAPI;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.util.AccessorUtil;
import org.ow2.bonita.util.BonitaException;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.novabpm.util.Misc;
import org.ow2.novabpm.util.SimpleCallbackHandler;

/**
 * 
 * @author Thomas Gueze
 *
 */
public class MultiInstantiationApproval extends AbstractExample {

  private static final Logger LOG = Logger.getLogger(MultiInstantiationApproval.class.getName());
  private Map<String, String> logins = new HashMap<String, String>();

  private Map<String, String> votes = new HashMap<String, String>();
  private int numberOfAccepted;
  
  
  @Override
  public void afterInstantiate(ProcessInstanceUUID instanceUUID,
    ProcessDefinitionUUID processUUID, RuntimeAPI runtimeAPI,
    QueryRuntimeAPI queryRuntimeAPI, QueryDefinitionAPI queryDefinitionAPI)
    throws BonitaException {
    
    switch (numberOfAccepted) {
      case 0:
        votes.put("john", "no");
        votes.put("jack", "no");
        votes.put("james", "no");
        break;
      case 1 :
        votes.put("john", "yes");
        votes.put("jack", "no");
        votes.put("james", "no");
        break;
      case 2 :
        votes.put("john", "yes");
        votes.put("jack", "yes");
        votes.put("james", "no");
        break;
      default:
      case 3 :
        votes.put("john", "yes");
        votes.put("jack", "yes");
        votes.put("james", "yes");
        break;
    }
    try {
      while (!votes.isEmpty()) {
        logout();
        String user = votes.keySet().iterator().next();
        String currentVote;
        if (logins.containsKey(user)) {
          login(user, logins.get(user));
          currentVote = votes.get(user);
        } else {
          String msg = "Please login as a separate user to start task. Tasks available for users: ";
          for (String v : votes.keySet()) {
            msg += v + " ";
          }
          System.out.println(msg);
          login();
          System.out.println("Will you accept the request ? [yes/no]");
          try {
            currentVote = new BufferedReader(new InputStreamReader(System.in)).readLine();
            if (!"yes".equals(currentVote) && !"no".equals(currentVote)) {
              throw new BonitaRuntimeException("invalid value " + currentVote);
            }
          } catch (IOException e) {
            throw new BonitaRuntimeException(e);
          }
        }
        ///// Find current user
        // TODO: find a better solution
        Collection<ActivityInstance<TaskInstance>> userTasks = 
          AccessorUtil.getAPIAccessor().getQueryRuntimeAPI().getTaskList(TaskState.READY);
        if (userTasks == null || userTasks.isEmpty()) {
          throw new BonitaRuntimeException("No task found for current user.");
        }
        String currentUser = userTasks.iterator().next().getBody().getTaskUser();
        votes.remove(currentUser);
        //// End of find current user
        performTask(instanceUUID,
          AccessorUtil.getAPIAccessor().getRuntimeAPI(), 
          AccessorUtil.getAPIAccessor().getQueryRuntimeAPI(), currentVote);
      }
    } catch (LoginException e) {
      throw new BonitaRuntimeException(e);
    }
  }
  
  protected void setLoginContext(LoginContext loginContext) {
    this.loginContext = loginContext;
  }
  
  public void initLogins() {
    logins.clear();
    logins.put("john", "bpm");
    logins.put("jack", "bpm");
    logins.put("james", "bpm");
  }
  
  public void performTask(ProcessInstanceUUID instanceUUID, RuntimeAPI runtimeAPI,
    QueryRuntimeAPI queryRuntimeAPI, String vote)
    throws BonitaException {
    final Collection<ActivityInstance< TaskInstance >> activities = queryRuntimeAPI.getTaskList(instanceUUID, TaskState.READY);

    if (activities.isEmpty()) {
      throw new BonitaRuntimeException("No task found? Bad User?");
    }
    for (ActivityInstance< TaskInstance > activity : activities) {
      final TaskUUID taskUUID = activity.getBody().getUUID();
      final String activityId = activity.getActivityId();
      if (LOG.isLoggable(Level.INFO)) {
        LOG.info(Misc.LINE_SEPARATOR 
            + "***** Starting task associated to activity: " + activityId + " *****" 
            + Misc.LINE_SEPARATOR);
      }
      runtimeAPI.startTask(taskUUID, true);
      runtimeAPI.setActivityInstanceVariable(activity.getUUID(),
          "decisionAccepted", vote);
      if (LOG.isLoggable(Level.INFO)) {
        LOG.info(Misc.LINE_SEPARATOR + "***** Finishing task associated to activity: " + activityId 
            + ": vote = " + vote + " *****"
            + Misc.LINE_SEPARATOR);
      }
      runtimeAPI.finishTask(taskUUID, true);
      if (LOG.isLoggable(Level.INFO)) {
        LOG.info(Misc.LINE_SEPARATOR 
            + "***** Task associated to activity: " + activityId + " finished. *****"
            + Misc.LINE_SEPARATOR);
      }
    }
  }

  @Override
  public String getProcessId() {
    return "multiInstantiation";
  }
  
  
  protected int getNumberOfAccepted() {
    return numberOfAccepted;
  }

  
  protected void setNumberOfAccepted(int numberOfAccepted) {
    this.numberOfAccepted = numberOfAccepted;
  }

  public void login(String userName, String password) throws LoginException {
    loginContext = new LoginContext("Bonita", new SimpleCallbackHandler(userName, password));
    loginContext.login();
  }
  
  public static void main(final String[] args) throws Exception {
    if (args == null || args.length != 2) {
      LOG.severe("Usage: java " + MultiInstantiationApproval.class.getName() + " barFile numberOfAccept");
      throw new BonitaRuntimeException("Please read the above usage");
    }
    final String barFile = args[0];
    
    final MultiInstantiationApproval approvalWorkflow = new MultiInstantiationApproval();
    approvalWorkflow.login();
    approvalWorkflow.setNumberOfAccepted(0);
    approvalWorkflow.execute(
        AccessorUtil.getAPIAccessor().getRuntimeAPI(),
        AccessorUtil.getAPIAccessor().getQueryRuntimeAPI(),
        AccessorUtil.getAPIAccessor().getManagementAPI(),
        AccessorUtil.getAPIAccessor().getQueryDefinitionAPI(),
        barFile, null);
    approvalWorkflow.logout();
  }
}
